/*
 *  GNOM - packet radio-terminal for Linux
 *  Copyright (C) 1997 Jochen Sarrazin, DG6VJ. All rights reserved.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "global.h"

#include "startup.h"
#include "comm.h"
#include "keyboard.h"
#include "display.h"
#include "sndqueue.h"
#include "savebuf.h"



void calc_fd_mask(fd_set *fd_mask)
{
  s_hardware *tmp;

  FD_ZERO(fd_mask);
  FD_SET(0,fd_mask);		/* Tastatur */
  
  tmp = hardware;
  while (tmp != NULL)
  {
    FD_SET(tmp->fd,fd_mask);
    tmp = tmp->next;
  }
}



int main()
{
  int ready=0;
  fd_set fd_mask;
  s_hardware *hardtmp;
  struct timeval tv;


  /* Copyright-Meldung */
  printf("GNOM/Linux V%i.%i.%i (%s)\nCopyright (C) 1997 Jochen Sarrazin. All rights reserved.\n",
         MAJORLEVEL,MINORLEVEL,PATCHLEVEL,DATUM);


  printf("- Reading gnom.conf...\n");
  if (read_gnom_conf() == FALSE) _exit(1);

  /* Ins GNOM-Verzeichnis wechseln */
  chdir(config->maindir);

  init_globals();

  /* Ntige Verzeichnisse anlegen */
  make_dirs();

  printf("- initializing scrollback-buffers for %i channels...\n",chancount);
  init_scrollbuffers();

  printf("- setting mycalls...\n");
  startup_set_mycalls();

  printf("- initializing hardware-drivers...\n");
  comm_init_drivers();

  printf("- initialize keyboard...\n");
  InitKeyboard();

  printf("- initialize display...\n");
  display_init();



  signal(SIGCLD,SIG_IGN);


  while (!ready) {
    calc_fd_mask(&fd_mask);

    tv.tv_sec = 1;
    tv.tv_usec = 0;

    if (select(maxfd,&fd_mask,NULL,NULL,&tv))
    {
      if (FD_ISSET(0,&fd_mask))
      {
        if (!DoKeyboard()) ready = 1;
        update_lower_statusline(TRUE);
      }

      hardtmp = hardware;
      while (hardtmp != NULL)
      {
        if (FD_ISSET(hardtmp->fd,&fd_mask))
        {
          /* Ja, der Treiber hier will was von uns */
          comm_driver_command(hardtmp);
        }
        hardtmp = hardtmp->next;
      }
    }
    else update_lower_statusline(TRUE);
  }


  /* Den Bildschirm deinitialisieren */
  display_deinit();

  printf("- save channel-status...");
  if (save_channelstatus())
     printf(" success\n");
  else
     printf(" failed\n");

  printf("- deinitialize keyboard...\n");
  Deinit_Keyboard();
  
  return(0);
}


