/*
 *  GNOM - packet radio-terminal for Linux
 *  Copyright (C) 1997 Jochen Sarrazin, DG6VJ. All rights reserved.
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/****************************************************************************/
/**  Funktionen fr die Passwort-Beantwortung                              **/
/****************************************************************************/


/* Mgliche pwmode's:
 *
 *  1 = RMNC-Passwort
 *  2 = BayCom-Box-Passwort
 *  3 = MD2
 *  4 = altes DieBox-Passwort
 *  5 = "neues" DieBox-Passwort
 *
 */


#include "global.h"
#include "password.h"

#include "toolbox.h"
#include "utils.h"
#include "md2.h"
#include "keyboard.h"
#include "sndqueue.h"

#include <string.h>


/* Holt sich die Passwort-Infos und schreibt sie nach chaninfo[]->pwinfo */
int pw_getinfo(int chan)
{
  s_udata udata;
  
  /* Infos ber diesen User aus der Datenbank holen */
  strcpy(udata.call,chaninfo[chan]->call);
  if (GetUserData(&udata) == 0) return(FALSE);
  
  if ((udata.pwmode < 0) || (udata.pwmode > 5)) return(FALSE);
  
  if (chaninfo[chan]->pwinfo == NULL)
     chaninfo[chan]->pwinfo = (s_pwinfo *)malloc(sizeof(s_pwinfo));
  
  chaninfo[chan]->pwinfo->pwmode = udata.pwmode;
  chaninfo[chan]->pwinfo->dbtime = NULL;
  chaninfo[chan]->pwinfo->pw = NULL;
  
  switch (udata.pwmode)
  {
    case 1: /* RMNC-PW */
            chaninfo[chan]->pwinfo->rmnc_pw = atoi(udata.password);
            break;
    case 2: /* BayBox-PW */
            chaninfo[chan]->pwinfo->pw = (char *)strdup(udata.password);
            break;
    case 3: /* MD2 */
            chaninfo[chan]->pwinfo->pw = (char *)strdup(udata.password);
            break;
    case 4: /* Altes DieBox-Verfahren */
            chaninfo[chan]->pwinfo->pw = (char *)strdup(udata.password);
            break;
    case 5: /* "Neues" DieBox-Verfahren */
            chaninfo[chan]->pwinfo->pw = (char *)strdup(udata.password);
            break;
  }
  
  return(TRUE);
}


void pw_free(int chan)
{
  if (chaninfo[chan]->pwinfo == NULL) return;
  
  if (chaninfo[chan]->pwinfo->dbtime != NULL)
     free(chaninfo[chan]->pwinfo->dbtime);

  switch (chaninfo[chan]->pwinfo->pwmode)
  {
    case 2: free(chaninfo[chan]->pwinfo->pw); break;
    case 3: free(chaninfo[chan]->pwinfo->pw); break;
  }
  free(chaninfo[chan]->pwinfo);
  chaninfo[chan]->pwinfo = NULL;
}



void password(int chan)
{
  /* Infos aus der User-Datenbank holen */
  if (chaninfo[chan]->pwinfo == NULL)
  {
    ErrorText("No password-informations available.",TRUE);
    return;
  }
  
  
  switch (chaninfo[chan]->pwinfo->pwmode)
  {
    case 1: /* FlexNet-Passwort. "SYS" abschicken */
            send_Textline(chan,"SYS\r");
            chaninfo[chan]->consstat = 2;
            chaninfo[chan]->flags &= ~CH_LINEMODE;
            break;
    case 2: /* BayBox-Passwort. "SYS" abschicken */
            send_Textline(chan,"SYS\r");
            chaninfo[chan]->consstat = 5;
            break;
    case 3: /* MD2-Password. "MD2" abschicken */
            send_Textline(chan,"MD2\r");
            chaninfo[chan]->consstat = 3;
            break;
    case 4: /* Altes DieBox-Passwort */
            pw_old_diebox_answer(chan);
            break;
    case 5: /* Neues DieBox-Passwort */
            pw_new_diebox_answer(chan);
            break;
  }
  
}



void pw_check_rmncpw(int chan, char *rxstr)
{
  int i;
  unsigned short answer;
  unsigned short sendback;
  char tmp[10];


  if ((i = POS(')',rxstr)) != -1)
     COPY(rxstr,rxstr,i+1,strlen(rxstr)-i-1);
  if ((i = POS('>',rxstr)) != -1)
     rxstr[i] = '\0';

  answer = atoi(rxstr);


  sendback = ((answer%10) * (chaninfo[chan]->pwinfo->rmnc_pw%10)) +
              (((answer/10)%10) * ((chaninfo[chan]->pwinfo->rmnc_pw/10)%10)) +
              (((answer/100)%10) * ((chaninfo[chan]->pwinfo->rmnc_pw/100)%10)) +
              (((answer/1000)%10) * ((chaninfo[chan]->pwinfo->rmnc_pw/1000)%10)) +
              (((answer/10000)%10) * ((chaninfo[chan]->pwinfo->rmnc_pw/10000)%10));


  sprintf(tmp,"%i\r",sendback);
  send_Textline(chan,tmp);
  
  chaninfo[chan]->flags |= CH_LINEMODE;
  
  chaninfo[chan]->consstat = 0;
}


void pw_calc_randomstring(int chan, int len, char *str)
{
  int i=0;
  int zahl,slen;

  slen = strlen(chaninfo[chan]->pwinfo->pw);

  while (i < len)
  {

    /* Wieviele Bytes kommen vor den Zufallsstring? */  
    srand(rand());
    zahl = 1+(int) ((float)slen*rand()/(RAND_MAX+1.0));
    
    str[i] = chaninfo[chan]->pwinfo->pw[zahl-1];
    
    i++;
  }
  str[len] = '\0';
}



/* Guckt, ob das ein BayCom-Passwort ist */
void pw_check_baycom(int chan, char *rxstr)
{
  char tmp[500],tmp2[500],answer[6]="";
  int i,k;
  
  if ((i = POS('>',rxstr)) == -1) return;
  
  COPY(tmp,rxstr,0,i);
  COPY(rxstr,rxstr,i+1,strlen(rxstr)-i-1);
  
  CutSSID(tmp2,chaninfo[chan]->call);
  
  /* Das ist nicht das Rufzeichen der Mailbox */
  if (strcmp(tmp2,tmp)) return;
  
  /**** Sooo - jetzt die 5 Zahlen 'rausfinden. Dazu erstmal das Passwort 'raussuchen */
  if (chaninfo[chan]->pwinfo == NULL)
  {
    ErrorText("No password-informations available.",TRUE);
    return;
  }
  
  KillSpacesLeft(rxstr);
  KillSpacesRight(rxstr);
  while (rxstr[0] != '\0')
  {
    if ((i = POS(' ',rxstr)) == -1)
    {
      strcpy(tmp,rxstr);
      rxstr[0] = '\0';
    }
    else
    {
      COPY(tmp,rxstr,0,i);
      COPY(rxstr,rxstr,i+1,strlen(rxstr)-i-1);
    }
    
    k = atoi(tmp);
    if (k == 0) return;
    if (k > strlen(chaninfo[chan]->pwinfo->pw)+1)
    {
      ErrorText("Wrong password.",TRUE);
      return;
    }
    sprintf(answer,"%s%c",answer,chaninfo[chan]->pwinfo->pw[k-1]);
  }
  

  /* Wieviele Bytes kommen vor den Zufallsstring? */  
  srand(rand());
  k = 1+(int) ((float)69*rand()/(RAND_MAX+1.0));
  
  /* Und wieviele dahinter? */
  i = 70-k;
  
  /* Die Zufallsstrings erzeugen */
  pw_calc_randomstring(chan,k,tmp);
  pw_calc_randomstring(chan,i,tmp2);
  
  strcat(tmp,answer);
  strcat(tmp,tmp2);
  strcat(tmp,"\xD");
  send_Textline(chan,tmp);
  
  if (chaninfo[chan]->consstat == 5) chaninfo[chan]->consstat = 0;
  
}



void pw_check_md2pw(int chan, char *data)
{
  int i;
  char call[20],tmp[500];
  
  
  if ((i = POS('>',data)) != -1)
  {
    COPY(call,data,0,i);
    COPY(data,data,i+1,strlen(data)-i-1);
    KillSpacesRight(call);
    Gross(call);
  }
  else return;
  
  CutSSID(tmp,chaninfo[chan]->call);
  if (strcmp(chaninfo[chan]->call,call) && strcmp(tmp,call)) return;
  
  if ((i = POS('[',data)) == -1) return;
  COPY(data,data,i+1,strlen(data)-i-1);
  if ((i = POS(']',data)) == -1) return;
  data[i] = '\0';

  /* Antwort berechnen */
  calc_MD2_pw(data,chaninfo[chan]->pwinfo->pw,tmp);
  
  strcat(tmp,"\r");
  send_Textline(chan,tmp);  

  chaninfo[chan]->consstat = 0;
}



void pw_check_for_dieboxtime(int chan,char *data)
{
  char *found,dummy[10];
  struct tm stm;

  if ((found = (char *)strstr(data,"Login: ")) == NULL) return;

  stm.tm_mday = -1;
  stm.tm_mon = -1;
  stm.tm_year = -1;
  stm.tm_hour = -1;
  stm.tm_min = -1;


  found += 7;			/* Den String "Login: " berspringen */
  
  /* Der briggeblibene String mu mindestens 14 Bytes lang sein */
  if (strlen(found) < 14) return;
  
  /* Tag */
  COPY(dummy,found,0,2);
  if ((stm.tm_mday = atoi(dummy)) == 0) return;
  /* Monat */
  COPY(dummy,found,3,2);
  if ((stm.tm_mon = atoi(dummy)) == 0) return;
  /* Jahr */
  COPY(dummy,found,6,2);
  if ((stm.tm_year = atoi(dummy)) == 0) return;
  /* Stunden */
  COPY(dummy,found,9,2);
  stm.tm_hour = atoi(dummy);
  /* Monat */
  COPY(dummy,found,12,2);
  stm.tm_min = atoi(dummy);
  
  chaninfo[chan]->pwinfo->dbtime = (struct tm *)malloc(sizeof(struct tm));
  chaninfo[chan]->pwinfo->dbtime->tm_mday = stm.tm_mday;
  chaninfo[chan]->pwinfo->dbtime->tm_mon = stm.tm_mon;
  chaninfo[chan]->pwinfo->dbtime->tm_year = stm.tm_year;
  chaninfo[chan]->pwinfo->dbtime->tm_hour = stm.tm_hour;
  chaninfo[chan]->pwinfo->dbtime->tm_min = stm.tm_min;

}



void pw_old_diebox_answer(int chan)
{
  int offs,multipl;
  struct tm *time;
  char tmp[500],data[5];
  int fd,i;
  
  time = chaninfo[chan]->pwinfo->dbtime;
  
  if (time == NULL)
  {
    ErrorText("No DieBox-Timeinfo received.",TRUE);
    return;
  }
  
  sprintf(tmp,"%s/%s",config->maindir,chaninfo[chan]->pwinfo->pw);
  i = filesize(tmp);
  if (i == -1)
  {
    ErrorText("Password-datafile is not available.",TRUE);
    return;
  }
  switch (i)
  {
    case 1620: /* File ohne CR/LF */
               multipl = 27;
               break;
    case 1680: /* File mit CR oder LF */
               multipl = 28;
               break;
    case 1740: /* File mit CR und LF */
               multipl = 29;
               break;
    default: /* Falsche Lnge */
             ErrorText("Password-File has an invalid length.",TRUE);
             return;
  }
  
  offs = time->tm_mday+time->tm_min;
  if (offs > 59) offs = offs - 60;
  offs = offs * multipl + time->tm_hour;
  
  if ((fd = open(tmp,O_RDONLY)) == -1)
  {
    ErrorText("Cannot open password-file",TRUE);
    return;
  }
  lseek(fd,offs,SEEK_SET);
  read(fd,data,4);
  data[4] = '\0';
  close(fd);
  
  sprintf(tmp,"PRIV %s\r",data);
  send_Textline(chan,tmp);
}



void pw_new_diebox_answer(int chan)
{
  int multipl;
  struct tm *time;
  char tmp[500],data[5];
  int fd,i;
  
  time = chaninfo[chan]->pwinfo->dbtime;

  ErrorText("Sorry, das neue DieBox-Verfahren ist noch nicht implementiert.",TRUE);
  return;
  
  if (time == NULL)
  {
    ErrorText("No DieBox-Timeinfo received.",TRUE);
    return;
  }
  
  sprintf(tmp,"%s/%s",config->maindir,chaninfo[chan]->pwinfo->pw);
  i = filesize(tmp);
  if (i == -1)
  {
    ErrorText("Password-datafile is not available.",TRUE);
    return;
  }
  switch (i)
  {
    case 1620: /* File ohne CR/LF */
               multipl = 27;
               break;
    case 1680: /* File mit CR oder LF */
               multipl = 28;
               break;
    case 1740: /* File mit CR und LF */
               multipl = 29;
               break;
    default: /* Falsche Lnge */
             ErrorText("Password-File has an invalid length.",TRUE);
             return;
  }
  
  /* Jetzt werden die 5 Offsets berechnet */


  
  
  if ((fd = open(tmp,O_RDONLY)) == -1)
  {
    ErrorText("Cannot open password-file",TRUE);
    return;
  }
  lseek(fd,3,SEEK_SET);
  read(fd,data,4);
  data[4] = '\0';
  close(fd);
  
  sprintf(tmp,"PRIV %s\r",data);
  send_Textline(chan,tmp);
}


