/****************************************************************************/
/**    Kernel-AX25-Treiber fr GNOM.                                       **/
/**  Copyright (C) 1997 Jochen Sarrazin, DG6VJ. All righs reserved.        **/
/**                                                                        **/
/** Header-File:                                                           **/
/**  Routinen fr die Kommunikation mit GNOM                               **/
/****************************************************************************/

#ifndef COMM_H
#define COMM_H

#include "global.h"
/*#include <sys/types.h>	/* socket() */
/*#include <sys/socket.h	/* socket() */


/*
###########################################################################
##   Implementierte Befehle                                              ##
###########################################################################
##  GNOM -> Treiber:                                                     ##
##   #1: Request driver Informations. MUSS der erste Befehl sein         ##
##   #2: Connect-Request - GNOM mchte gerne connecten                   ##
##   #5: Disconnect-Request                                              ##
##   #6: Daten                                                           ##
##   #7: Listen to call (sowas wie MYCALL)                               ##
##   #8: Don't listen to call (nicht mehr auf das Rufzeichen hren)      ##
##  #10: TxDelay setzen                                                  ##
##  #11: Persistence setzen                                              ##
##  #12: Frack-Time setzen                                               ##
##  #13: Retry                                                           ##
##  #14: Maxframe                                                        ##
##  #15: TxTail                                                          ##
##  #16: Digipeat                                                        ##
##  #17: Response-Time                                                   ##
##  #18: Idle-Time                                                       ##
##  #50: Monitor aktivieren                                              ##
##  #51: Monitor deaktivieren                                            ##
##                                                                       ##
##  Treiber -> GNOM:                                                     ##
##   #2: Antwort auf einen Connect-Request                               ##
##   #3: Ein Connect kommt von auen                                     ##
##   #4: Status-nderungen                                               ##
##   #5: Disconnect                                                      ##
##   #6: Daten                                                           ##
##   #7: Alle gespeicherten QSO-Daten wurden abgeschickt.                ##
##  #10: Antwort TxDelay                                                 ##
##  #11: Antwort Persistence                                             ##
##  #12: Antwort Frack-Time                                              ##
##  #13: Antwort Retry                                                   ##
##  #14: Antwort Maxframe                                                ##
##  #15: Antwort TxTail                                                  ##
##  #16: Antwort Digipeat                                                ##
##  #17: Antwort Response-Time                                           ##
##  #18: Antwort Idle-Time                                               ##
##  #40: Es dard ein Datenframe versendet werden                         ##
##  #41: Stop, jetzt kann ich keine Daten mehr aufnehmen                 ##
##  #50: Monitor-Daten (siehe s_monitor)                                 ##
###########################################################################
*/





/*
 * Diese Infos werden an GNOM gesendet, wenn von auen ein connect 'reingekommen ist
 */
typedef struct
{
  char port[20];		/* Portname */
  char call[10];		/* Rufzeichen */
  char path[100];		/* Pfad */
  char mycall[10];		/* eigenes Rufzeichen */
  int qsonr;			/* QSO-Nummer in diesem Treiber */
  int old;
} s_got_connect;



/*
 * Das hier kommt von GNOM, wenn ein Connect gewnscht wird
 */
typedef struct
{
  char port[20];
  char mycall[10];
  char call[10];
  char digis[100];
  int chan;			/* Kanal-Nummer vom Gnom */
} s_make_connect;



/*
 * Diverse Informationen aus dem Monitor
 * Bitte die Formate, vor allem die der Strings, 100%ig einhalten!
 *
 * Nach diesen Header-Informationen kommen die einzelnen Zeilen ohne CR dahinter.
 * Format: 4 Bytes lnge (int), dann der Text.
 *
 * Als Daten drfen gerne schon ausgewertete Frames (z.B. NetRom, TCP/IP etc) kommen.
 * Es mu nicht der Plaintext sein.
 */
#define MONI_DAMA 		1	/* Ist das Frame DAMA? */
#define MONI_AX_EAX 		2	/* AX25 oder EAX25? */
#define MONI_POLLFINAL 		4	/* Ist das Poll/Final-Bit gesetzt? */
#define MONI_BAD_HEADER 	8	/* Bad header? */
typedef struct
{
  char port[20];		/* Auf welchem Port wurde empfangen? */
  uint flags;			/* siehe MONI_* - Defines */
  char calltocall[30];		/* z.B. "DG6VJ-10->DG4IAD-12" */
  char digis[100];		/* z.B. "DB0EA DB0CPU*" */
  int frametype;		/*  1 = SABM
                                 *  2 = UA
                                 *  3 = I-Frame
                                 *  4 = RR
                                 *  5 = RNR
                                 *  6 = U
                                 *  7 = DISC
                                 *  8 = DM
                                 *  9 = FRMR
                                 * 10 = UI
                                 * 11 = SABME
                                 * 12 = REJ
                                 */
  int cmd_resp;			/* 1 = Command, 2 = Response, 0 = weder noch */
  int pid;			/* Die PID. Bitte die Zahl, einen Text macht GNOM draus. */
  unsigned char rxcount;	/* RX-Frame-Counter-Position (0-7) */
  unsigned char txcount;	/* TX-Frame-Counter-Position (0-7) */
  int anz_datalines;		/* Wieviele Datenzeilen kommen nach diesem Header? */
  char frmr_info[100];		/* Nhere Infos zu der frmr-sache */
} s_monitor;



/*
 * Auf diesem Port soll auf ein bestimmtes Rufzeichen gehrt werden
 */
typedef struct
{
  char port[20];
  char call[10];
} s_listen_to;


/************************************************************************************/
/********* FUNKTIONS-PROTOTYPEN *****************************************************/
/************************************************************************************/


/* Die Gegenseite hat disconnected, bevor die Daten abgeschickt wurden.
 * Alle Daten lschen
 */
void comm_delete_saved_data(int qsonr);


/*
 * Erzeugt einen Socket, und lauscht da
 */
void comm_create_and_listen_to_socket();



/*
 * Treiber-Informationen an GNOM senden. Vor diesen Daten darf kein Befehl
 * an GNOM gesendet werden!
 */
void comm_send_driver_informations();



/*
 * Ein disconnect-request von GNOM wird bearbeitet
 */
void comm_get_disconect_request();



/*
 * Antwort auf einen Connect-Versuch.
 * Code 1  = invalid parameter
 *      2  = cannot open socket
 *      3  = cannot bind socket
 *      4  = cannot connect twice
 *      FF = ok
 *
 * In chan wird der GNOM-Kanal zurckgegeben, damit GNOM die Zuordnung Kanal<->qsonr machen kann.
 */
void comm_connect_answer(uchar code, int qsonr, int chan);



/*
 * Irgendwas hat sich gendert. Der Status, die Anzahl Frames in der Send-Queue
 * (Unsent + Unack), die Anzahl Retries...
 */
void comm_poll_change(int qsonr, uchar what);



/*
 * von auen kommt ein connect-wunsch 'rein
 */
void comm_get_connect_request();



/*
 * Sendet ein Frame von GNOM in Richtung Kernel-AX25 aus.
 */
void comm_get_senddata();



/*
 * Holt einen Befehl von GNOM und wertet ihn aus.
 */
void comm_check_command();



/*
 * Sagt GNOM bescheid, da jemand connectet hat
 */
void comm_notice_connect(int qsonr);



/*
 * GNOM bescheid sagen, da die Gegenseite disconnected hat
 */
void comm_notice_disconect(int qsonr);



/*
 * Diese Daten gehen in Richtung GNOM.
 *
 * Wenn GNOM nicht luft, werden die Daten lokal zwischengespeichert.
 */
void comm_send_data(int qsonr, char *data, int count);



/*
 * GNOM hat sich grade gemeldet. Gespeicherte Daten werden abgesendet.
 */
void comm_send_saved_data(int qsonr);



/*
 * Schickt alle QSOs an GNOM.
 */
void comm_send_allqsos();



/* Txdelay einstellen */
void comm_set_txdelay();

/* Persistance einstellen */
void comm_set_persis();

/* Frack-Time einstellen */
void comm_set_frack();

/* Retry-Time einstellen */
void comm_set_retry();

/* Maxframe einstellen */
void comm_set_maxframe();

/* txtail einstellen */
void comm_set_txtail();

/* digipeating? */
void comm_set_digipeat();

/* response-time einstellen */
void comm_set_response();

/* idle-time einstellen */
void comm_set_idle();


void comm_i_want_a_frame(int qsonr);


void comm_i_dont_want_a_frame(int qsonr);


#endif

